package com.diagbot.rule;

import com.diagbot.dto.BillMsg;
import com.diagbot.dto.RuleBaseDTO;
import com.diagbot.dto.RuleSimpleDTO;
import com.diagbot.enums.RedisEnum;
import com.diagbot.util.CoreUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.util.MsgUtil;
import com.diagbot.util.RedisUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;

/**
 * @description: 实验室检查和辅助检查规则
 * @author: kwz
 * @time: 2020/8/3 14:47
 */
@Component
public class DrugRule {

    @Autowired
    RedisUtil redisUtil;

    /**
     * 比较药品是否匹配(先根据baseLibType判断返回的是药品通用名还是药品分类，如果返回的是药品分类，从redis获取药品通用名)
     *
     * @param inputList
     * @param ruleBaseDTO
     * @param billMsgList
     * @param conType
     * @param ruleSimpleDTO
     * @param <T>
     */
    public <T> void bill(List<T> inputList, RuleBaseDTO ruleBaseDTO, List<BillMsg> billMsgList, String conType,
                         RuleSimpleDTO ruleSimpleDTO) {
        if (ListUtil.isNotEmpty(inputList)) {
            List<String> allDrug = new ArrayList<>();
            if (ruleBaseDTO.getBaseLibType().equals(301) || ruleBaseDTO.getBaseLibType().equals(302)
                    || ruleBaseDTO.getBaseLibType().equals(303) || ruleBaseDTO.getBaseLibType().equals(304)) {
                List<String> drugRedis = redisUtil.get(RedisEnum.drugType.getName() + ruleBaseDTO.getBaseLibName());
                if (ListUtil.isNotEmpty(drugRedis)) {
                    allDrug.addAll(drugRedis);
                }
            } else {
                allDrug.add(ruleBaseDTO.getBaseLibName());
            }
            for (T d : inputList) {
                String uniqueName = (String) CoreUtil.getFieldValue(d, "uniqueName");
                String name = (String) CoreUtil.getFieldValue(d, "name");
                if (StringUtils.isNotBlank(uniqueName) && allDrug.contains(uniqueName)) {
                    String message = "";
                    if ("禁忌过敏药品".equals(conType)) {
                        message = name + "过敏";
                    } else {
                        message = "可能正在用药" + name;
                    }
                    BillMsg commonBillMsg = MsgUtil.getCommonBillMsg(
                            ruleSimpleDTO.getInputName(), ruleSimpleDTO.getLibName(),
                            message, conType, ruleSimpleDTO.getLibTypeName());
                    billMsgList.add(commonBillMsg);
                }
            }
        }
    }
}
